package me.desht.pneumaticcraft.common.ai;

import me.desht.pneumaticcraft.api.item.IItemRegistry.EnumUpgrade;
import me.desht.pneumaticcraft.common.entity.living.EntityDrone;
import me.desht.pneumaticcraft.common.item.Itemss;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.ai.EntityAIAttackMelee;

public class DroneAIAttackEntity extends EntityAIAttackMelee {
    private final EntityDrone attacker;
    private final boolean isRanged;
    private final double rangedAttackRange;

    public DroneAIAttackEntity(EntityDrone attacker, double speed, boolean useLongMemory) {
        super(attacker, speed, useLongMemory);
        this.attacker = attacker;
        isRanged = attacker.hasMinigun();
        rangedAttackRange = 16 + Math.min(16, attacker.getUpgrades(Itemss.upgrades.get(EnumUpgrade.RANGE)));
    }

    @Override
    public boolean shouldExecute() {
        if (isRanged && attacker.getAmmo() == null) {
            attacker.addDebugEntry("gui.progWidget.entityAttack.debug.noAmmo");
            return false;
        }

        EntityLivingBase entitylivingbase = attacker.getAttackTarget();
        if (entitylivingbase == null) {
            attacker.addDebugEntry("gui.progWidget.entityAttack.debug.noEntityToAttack");
        }

        return super.shouldExecute();
    }

    @Override
    public boolean shouldContinueExecuting() {
        if (isRanged) {
            EntityLivingBase entitylivingbase = attacker.getAttackTarget();
            if (entitylivingbase == null) return false;
            double dist = attacker.getDistanceSq(entitylivingbase.posX, entitylivingbase.getEntityBoundingBox().minY, entitylivingbase.posZ);
            if (attacker.getAmmo() == null) return false;
            if (dist < Math.pow(rangedAttackRange, 2) && attacker.getEntitySenses().canSee(entitylivingbase))
                return true;
        }
        return super.shouldContinueExecuting();
    }

    @Override
    public void resetTask() {

    }

    @Override
    public void updateTask() {
        boolean needingSuper = true;
        if (isRanged) {
            EntityLivingBase entitylivingbase = attacker.getAttackTarget();
            double dist = attacker.getDistanceSq(entitylivingbase.posX, entitylivingbase.getEntityBoundingBox().minY, entitylivingbase.posZ);
            if (dist < Math.pow(rangedAttackRange, 2) && attacker.getEntitySenses().canSee(entitylivingbase)) {
                attacker.getFakePlayer().posX = attacker.posX;//Knockback direction
                attacker.getFakePlayer().posY = attacker.posY;
                attacker.getFakePlayer().posZ = attacker.posZ;
                attacker.tryFireMinigun(entitylivingbase);
                needingSuper = false;
                if (dist < Math.pow(rangedAttackRange - 4, 2)) {
                    attacker.getNavigator().clearPath();
                }
            }
        }
        if (needingSuper) super.updateTask();
    }
}
